@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2013 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -15,16 +15,17 @@
 
 
 
+
+
  dtheta.deta <-
   function(theta,
-           link = "identity",
-           earg = list(theta = theta, # Needed
-                       inverse = FALSE,
+           link = "identitylink",
+           earg = list(theta = theta,  # Needed
+                       inverse = TRUE,  # 20150711: big change!!!!
                        deriv = 1,
                        short = TRUE,
                        tag = FALSE)) {
 
-
   function.name  <- link
 
   function.name2 <- attr(earg, "function.name")
@@ -32,9 +33,13 @@
     warning("apparent conflict in name of link function")
   }
 
-  earg[["theta"]] <- theta # New data
+  earg[["theta"]] <- theta  # New data
 
-  earg[["deriv"]] <- 1 # New
+  if (length(earg$inverse))
+    earg[["inverse"]] <- TRUE else
+    earg$inverse <- TRUE
+
+  earg[["deriv"]] <- 1  # New
 
 
   do.call(what = function.name, args = earg)
@@ -44,11 +49,12 @@
 
 
 
- d2theta.deta2 <- 
+
+ d2theta.deta2 <-
   function(theta,
-           link = "identity",
-           earg = list(theta = theta, # Needed
-                       inverse = FALSE,
+           link = "identitylink",
+           earg = list(theta = theta,  # Needed
+                       inverse = TRUE,  # 20150711: big change!!!!
                        deriv = 2,
                        short = TRUE,
                        tag = FALSE)) {
@@ -60,18 +66,50 @@
   if (length(function.name2) && function.name != function.name2)
     warning("apparent conflict in name of link function in D2theta.deta2")
 
-  earg[["theta"]] <- theta # New data
+  earg[["theta"]] <- theta  # New data
 
-  earg[["deriv"]] <- 2 # New
+
+  if (length(earg$inverse))
+    earg[["inverse"]] <- TRUE else
+    earg$inverse <- TRUE
+
+  earg[["deriv"]] <- 2  # New
 
   do.call(what = function.name, args = earg)
 }
 
 
 
+
+
+ d3theta.deta3 <-
+  function(theta,
+           link = "identitylink",
+           earg = list(theta = theta,
+                       inverse = TRUE,
+                       deriv = 3,
+                       short = TRUE,
+                       tag = FALSE)) {
+
+  function.name  <- link
+  earg[["theta"]] <- theta  # New data
+
+  if (length(earg$inverse))
+    earg[["inverse"]] <- TRUE else
+    earg$inverse <- TRUE
+
+  earg[["deriv"]] <- 3  # New
+  do.call(what = function.name, args = earg)
+}
+
+
+
+
+
+
  theta2eta <-
   function(theta,
-           link = "identity",
+           link = "identitylink",
            earg = list(theta = NULL)) {
 
   function.name  <- link
@@ -80,7 +118,7 @@
   if (length(function.name2) && function.name != function.name2)
     warning("apparent conflict in name of link function")
 
-  earg[["theta"]] <- theta # New data
+  earg[["theta"]] <- theta  # New data
 
   do.call(what = function.name, args = earg)
 }
@@ -89,8 +127,8 @@
 
 
  eta2theta <-
-  function(theta, # This is really eta.
-           link = "identity",
+  function(theta,  # This is really eta.
+           link = "identitylink",
            earg = list(theta = NULL)) {
 
 
@@ -120,7 +158,7 @@
     stop("length(earg) == 0 not allowed")
 
 
-  if (llink == 1) { # ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
+  if (llink == 1) {  # ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
 
 
     if (is.list(earg[[1]]))
@@ -128,14 +166,14 @@
 
     function.name  <- link
 
-    function.name2 <- attr(earg, "function.name") # May be, e.g., NULL
+    function.name2 <- attr(earg, "function.name")  # May be, e.g., NULL
     if (length(function.name2) && function.name != function.name2)
       warning("apparent conflict in name of link function")
 
 
-    earg[["theta"]] <- theta # New data
+    earg[["theta"]] <- theta  # New data
 
-    earg[["inverse"]] <- TRUE # New
+    earg[["inverse"]] <- TRUE  # New
 
     return(do.call(what = function.name, args = earg))
   }
@@ -148,21 +186,21 @@
 
 
 
-   if (!is.matrix(theta) &&
-       length(theta) == length(earg))
-     theta <- rbind(theta)
+ if (!is.matrix(theta) &&
+     length(theta) == length(earg))
+   theta <- rbind(theta)
 
 
-    ans <- NULL
-    for(iii in 1:llink) {
-        use.earg <- earg[[iii]]
-        use.earg[["inverse"]] <- TRUE # New
-        use.earg[["theta"]] <- theta[, iii] # New
-        use.function.name <- link[iii]
+  ans <- NULL
+  for (iii in 1:llink) {
+    use.earg <- earg[[iii]]
+    use.earg[["inverse"]] <- TRUE  # New
+    use.earg[["theta"]] <- theta[, iii]  # New
+    use.function.name <- link[iii]
 
-        ans <- cbind(ans, do.call(what = use.function.name,
-                                  args = use.earg))
-      }
+    ans <- cbind(ans, do.call(what = use.function.name,
+                              args = use.earg))
+  }
 
   if (length(orig.earg) == ncol(ans) &&
       length(names(orig.earg)) > 0 &&
@@ -175,8 +213,33 @@
 
 
 
+
+
  namesof <- function(theta,
-                     link = "identity",
+                     link = "identitylink",
+                     earg = list(tag = tag, short = short),
+                     tag = FALSE,
+                     short = TRUE) {
+
+  funname.only <- strsplit(as.character(link), "(", fixed = TRUE)
+  funname.only <- (funname.only[[1]])[1]
+  link <- funname.only
+
+  earg[["theta"]] <- as.character(theta)
+
+  earg[["tag"]] <- tag
+  earg[["short"]] <- short
+
+
+  do.call(link, args = earg)
+}
+
+
+
+
+if (FALSE)
+ namesof <- function(theta,
+                     link = "identitylink",
                      earg = list(tag = tag, short = short),
                      tag = FALSE,
                      short = TRUE) {
@@ -204,7 +267,7 @@ link2list <- function(link
   big.list <- as.list(as.function(get(fun.name)))
 
 
-  big.list[[length(big.list)]] <- NULL # Kill the body of code
+  big.list[[length(big.list)]] <- NULL  # Kill the body of code
 
 
 
@@ -212,7 +275,7 @@ link2list <- function(link
 
   t.index <- pmatch(names(ans[-1]), names(big.list))
   t.index
-  if (any(is.na(t.index)))
+  if (anyNA(t.index))
     stop("in '", fun.name, "' could not match argument(s) ",
          paste('"', names(ans[-1])[is.na(t.index)], '"', sep = "",
                collapse = ", "))
