@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -9,28 +9,30 @@
 
 
 
-cao  <- function(formula,
-                 family, data = list(), 
-                 weights = NULL, subset = NULL, na.action = na.fail,
-                 etastart = NULL, mustart = NULL, coefstart = NULL,
-                 control = cao.control(...), 
-                 offset = NULL, 
-                 method = "cao.fit",
-                 model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                 contrasts = NULL, 
-                 constraints = NULL,
-                 extra = NULL, 
-                 qr.arg = FALSE, smart = TRUE, ...) {
+cao  <-
+  function(formula,
+           family = stop("argument 'family' needs to be assigned"),
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           etastart = NULL, mustart = NULL, coefstart = NULL,
+           control = cao.control(...),
+           offset = NULL,
+           method = "cao.fit",
+           model = FALSE, x.arg = TRUE, y.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = NULL,
+           qr.arg = FALSE, smart = TRUE, ...) {
   dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "cao"
 
   ocall <- match.call()
 
-  if (smart) 
+  if (smart)
     setup.smart("write")
 
   mt <- terms(formula, data = data)
-  if (missing(data)) 
+  if (missing(data))
     data <- environment(formula)
 
   mf <- match.call(expand.dots = FALSE)
@@ -39,9 +41,9 @@ cao  <- function(formula,
     mf$contrasts <- mf$constraints <- mf$extra <- mf$qr.arg <- NULL
   mf$coefstart <- mf$etastart <- mf$... <- NULL
   mf$smart <- NULL
-  mf$drop.unused.levels <- TRUE 
+  mf$drop.unused.levels <- TRUE
   mf[[1]] <- as.name("model.frame")
-  mf <- eval(mf, parent.frame()) 
+  mf <- eval(mf, parent.frame())
   if (method == "model.frame")
     return(mf)
   na.act <- attr(mf, "na.action")
@@ -58,12 +60,12 @@ cao  <- function(formula,
   x <- model.matrix(mt, mf, contrasts)
   attr(x, "assign") <- attrassigndefault(x, mt)
   offset <- model.offset(mf)
-  if (is.null(offset)) 
-    offset <- 0 # yyy ???
+  if (is.null(offset))
+    offset <- 0  # yyy ???
   w <- model.weights(mf)
-  if (!length(w))
-    w <- rep(1, nrow(mf))
-  else if (ncol(as.matrix(w)) == 1 && any(w < 0))
+  if (!length(w)) {
+    w <- rep_len(1, nrow(mf))
+  } else if (NCOL(w) == 1 && any(w < 0))
     stop("negative weights not allowed")
 
   if (is.character(family))
@@ -83,7 +85,7 @@ cao  <- function(formula,
   cao.fitter <- get(method)
 
 
-  deviance.Bestof <- rep(as.numeric(NA), len = control$Bestof)
+  deviance.Bestof <- rep_len(NA_real_, control$Bestof)
   for (tries in 1:control$Bestof) {
     if (control$trace && (control$Bestof > 1)) {
       cat(paste("\n========================= Fitting model",
@@ -165,7 +167,7 @@ cao  <- function(formula,
   } else list()  # R-1.5.0
 
   slot(answer, "iter") <- fit$iter
-  fit$predictors <- as.matrix(fit$predictors)  # Must be a matrix 
+  fit$predictors <- as.matrix(fit$predictors)  # Must be a matrix
   dimnames(fit$predictors) <- list(dimnames(fit$predictors)[[1]],
                                    fit$misc$predictors.names)
   slot(answer, "predictors") <- fit$predictors
