@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -36,18 +36,18 @@ Select <-
   if (is.character(exclude))
     if (any(nchar(prefix) == 0))
       stop("bad input for argument 'exclude'")
-  if (!is.logical(sort.arg) || 
+  if (!is.logical(sort.arg) ||
       length(sort.arg) != 1)
     stop("bad input for argument 'sort.arg'")
 
 
   col.names <- colnames(data)
   if (is.logical(prefix)) {
-    index <- if (prefix) 1:length(col.names) else
+    index <- if (prefix) seq_along(col.names) else
              stop("cannot have 'prefix = FALSE'")
   } else {
     index <- NULL
-    for (ii in 1:length(prefix)) {
+    for (ii in seq_along(prefix)) {
       small.col.names <- substr(col.names, 1, nchar(prefix[ii]))
 
       index <- c(index, grep(prefix[ii], small.col.names))
@@ -63,9 +63,9 @@ Select <-
 
   if (length(exclude)) {
     exclude.index <- NULL
-    for (ii in 1:length(exclude)) {
+    for (ii in seq_along(exclude)) {
       exclude.index <- c(exclude.index,
-                         (1:length(col.names))[exclude[ii] == col.names])
+                         (seq_along(col.names))[exclude[ii] == col.names])
     }
     exclude.index <- unique(sort(exclude.index))
     index <- setdiff(index, exclude.index)
@@ -185,7 +185,7 @@ subsetc <-
   if (!is.vector(vov))
     stop("argument 'vov' must be a vector")
   objvals <- vov
-  for (ii in 1:length(vov))
+  for (ii in seq_along(vov))
     objvals[ii] <- objfun(vov[ii], y = y, x = x, w = w,
                           extraargs = extraargs, ...)
   try.this <- if (abs.arg) {
@@ -210,6 +210,114 @@ subsetc <-
 
 
 
+ grid.search2 <-
+    function(vov1, vov2, objfun, y, x, w, extraargs = NULL,
+             maximize = TRUE,  # abs.arg = FALSE,
+             ret.objfun = FALSE, ...) {
+  if (!is.vector(vov1))
+    stop("argument 'vov1' must be a vector")
+  if (!is.vector(vov2))
+    stop("argument 'vov2' must be a vector")
+
+  allmat1 <- expand.grid(vov1 = as.vector(vov1),
+                         vov2 = as.vector(vov2))
+  objvals <- numeric(nrow(allmat1))
+  for (ii in seq_along(objvals))
+    objvals[ii] <- objfun(allmat1[ii, "vov1"],
+                          allmat1[ii, "vov2"],
+                          y = y, x = x, w = w,
+                          extraargs = extraargs, ...)
+
+  ind5 <- if (maximize) which.max(objvals) else which.min(objvals)
+
+
+  c(Value1 = allmat1[ind5, "vov1"],
+    Value2 = allmat1[ind5, "vov2"],
+    ObjFun = if (ret.objfun) objvals[ind5] else NULL)
+}
+
+
+
+
+ grid.search3 <-
+    function(vov1, vov2, vov3, objfun, y, x, w, extraargs = NULL,
+             maximize = TRUE,  # abs.arg = FALSE,
+             ret.objfun = FALSE, ...) {
+  if (!is.vector(vov1))
+    stop("argument 'vov1' must be a vector")
+  if (!is.vector(vov2))
+    stop("argument 'vov2' must be a vector")
+  if (!is.vector(vov3))
+    stop("argument 'vov3' must be a vector")
+
+  allmat1 <- expand.grid(vov1 = as.vector(vov1),
+                         vov2 = as.vector(vov2),
+                         vov3 = as.vector(vov3))
+  objvals <- numeric(nrow(allmat1))
+  for (ii in seq_along(objvals))
+    objvals[ii] <- objfun(allmat1[ii, "vov1"],
+                          allmat1[ii, "vov2"],
+                          allmat1[ii, "vov3"],
+                          y = y, x = x, w = w,
+                          extraargs = extraargs, ...)
+
+  ind5 <- if (maximize) which.max(objvals) else which.min(objvals)
+
+
+  c(Value1 = allmat1[ind5, "vov1"],
+    Value2 = allmat1[ind5, "vov2"],
+    Value3 = allmat1[ind5, "vov3"],
+    ObjFun = if (ret.objfun) objvals[ind5] else NULL)
+}
+
+
+
+
+ grid.search4 <-
+    function(vov1, vov2, vov3, vov4,
+             objfun, y, x, w, extraargs = NULL,
+             maximize = TRUE,  # abs.arg = FALSE,
+             ret.objfun = FALSE, ...) {
+  if (!is.vector(vov1))
+    stop("argument 'vov1' must be a vector")
+  if (!is.vector(vov2))
+    stop("argument 'vov2' must be a vector")
+  if (!is.vector(vov3))
+    stop("argument 'vov3' must be a vector")
+  if (!is.vector(vov4))
+    stop("argument 'vov4' must be a vector")
+
+  allmat1 <- expand.grid(vov1 = as.vector(vov1),
+                         vov2 = as.vector(vov2),
+                         vov3 = as.vector(vov3),
+                         vov4 = as.vector(vov4))
+  objvals <- numeric(nrow(allmat1))
+  for (ii in seq_along(objvals))
+    objvals[ii] <- objfun(allmat1[ii, "vov1"],
+                          allmat1[ii, "vov2"],
+                          allmat1[ii, "vov3"],
+                          allmat1[ii, "vov4"],
+                          y = y, x = x, w = w,
+                          extraargs = extraargs, ...)
+
+  ind5 <- if (maximize) which.max(objvals) else which.min(objvals)
+
+
+  c(Value1 = allmat1[ind5, "vov1"],
+    Value2 = allmat1[ind5, "vov2"],
+    Value3 = allmat1[ind5, "vov3"],
+    Value4 = allmat1[ind5, "vov4"],
+    ObjFun = if (ret.objfun) objvals[ind5] else NULL)
+}
+
+
+
+
+
+
+
+
+
 
 
  getind <- function(constraints, M, ncolx) {
@@ -229,7 +337,7 @@ subsetc <-
   temp2 <- matrix(unlist(constraints), nrow = M)
   for (kk in 1:M) {
     ansx <- NULL
-    for (ii in 1:length(constraints)) {
+    for (ii in seq_along(constraints)) {
       temp <- constraints[[ii]]
       isfox <- any(temp[kk, ] != 0)
       if (isfox) {
@@ -249,7 +357,7 @@ subsetc <-
 
  cm.VGAM <-
   function(cm, x, bool, constraints,
-           apply.int = FALSE, 
+           apply.int = FALSE,
            cm.default = diag(nrow(cm)),  # 20121226
            cm.intercept.default = diag(nrow(cm))  # 20121226
           ) {
@@ -272,7 +380,7 @@ subsetc <-
 
   if (!length(constraints)) {
     constraints <- vector("list", length(nasgn))
-    for (ii in 1:length(nasgn)) {
+    for (ii in seq_along(nasgn)) {
       constraints[[ii]] <- cm.default  # diag(M)
     }
     names(constraints) <- nasgn
@@ -280,7 +388,7 @@ subsetc <-
 
     if (any(nasgn == "(Intercept)"))
       constraints[["(Intercept)"]] <- cm.intercept.default
-  } 
+  }
 
   if (!is.list(constraints))
     stop("argument 'constraints' must be a list")
@@ -362,7 +470,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   if (!is.list(constraints))
     stop("'constraints' must be a list")
 
-  for (ii in 1:length(asgn))
+  for (ii in seq_along(asgn))
     constraints[[nasgn[ii]]] <- if (is.null(constraints[[nasgn[ii]]]))
       diag(M) else eval(constraints[[nasgn[ii]]])
 
@@ -379,19 +487,83 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
     stop("'nointercept' out of range")
   if (nasgn[1] != "(Intercept)" || M == 1)
     stop("Need an (Intercept) constraint matrix with M>1")
-  if (!all.equal(constraints[["(Intercept)"]], diag(M)))
+  if (!identical(constraints[["(Intercept)"]], diag(M)))
     warning("Constraint matrix of (Intercept) not diagonal")
 
   temp <- constraints[["(Intercept)"]]
   temp <- temp[, -nointercept, drop = FALSE]
-  constraints[["(Intercept)"]] <- temp 
+  constraints[["(Intercept)"]] <- temp
   constraints
 }
 
 
 
 
- cm.zero.VGAM <- function(constraints, x, zero = NULL, M = 1) {
+ cm.zero.VGAM <- function(constraints, x, zero = NULL, M = 1,
+                          predictors.names, M1 = 1) {
+
+
+  dotzero <- zero  # Transition
+
+  if (length(dotzero) == 1 && (dotzero == "" || is.na(dotzero)))
+    dotzero <- NULL
+
+  if (is.character(dotzero)) {
+
+
+
+
+    which.numeric.all <- NULL
+    for (ii in seq_along(dotzero)) {
+      which.ones <-
+          grep(dotzero[ii], predictors.names, fixed = TRUE)
+      if (length(which.ones)) {
+        which.numeric.all <- c(which.numeric.all, which.ones)
+      } else {
+        warning("some values of argument 'zero' are unmatched. Ignoring them")
+      }
+    }  # for ii
+    which.numeric <- unique(sort(which.numeric.all))
+
+    if (!length(which.numeric)) {
+      warning("No values of argument 'zero' were matched.")
+      which.numeric <- NULL
+    } else if (length(which.numeric.all) > length(which.numeric)) {
+      warning("There were redundant values of argument 'zero'.")
+    }
+
+    dotzero <- which.numeric
+  }  # if is.character(dotzero)
+
+
+
+  posdotzero <-  dotzero[dotzero > 0]
+  negdotzero <-  dotzero[dotzero < 0]
+
+
+  zneg.index <- if (length(negdotzero)) {
+
+    if (!is.Numeric(-negdotzero, positive = TRUE,
+                    integer.valued = TRUE) ||
+        max(-negdotzero) > M1)
+        stop("bad input for argument 'zero'")
+
+    bigUniqInt <- 1080
+    zneg.index <- rep(0:bigUniqInt, rep(length(negdotzero),
+                      1 + bigUniqInt)) * M1 + abs(negdotzero)
+    sort(intersect(zneg.index, 1:M))
+  } else {
+    NULL
+  }
+
+  zpos.index <- if (length(posdotzero)) posdotzero else NULL
+  z.Index <- if (!length(dotzero)) NULL else
+               unique(sort(c(zneg.index, zpos.index)))
+
+
+  zero <- z.Index  # Transition
+
+
 
   asgn <- attr(x, "assign")
   nasgn <- names(asgn)
@@ -402,17 +574,16 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   if (!is.list(constraints))
     stop("'constraints' must be a list")
 
-  for (ii in 1:length(asgn))
+  for (ii in seq_along(asgn))
     constraints[[nasgn[ii]]] <- if (is.null(constraints[[nasgn[ii]]]))
       diag(M) else eval(constraints[[nasgn[ii]]])
 
   if (is.null(zero))
     return(constraints)
 
-  if (!is.numeric(zero))
-    stop("argument 'zero' must be numeric")
   if (any(zero < 1 | zero > M))
-    stop("argument 'zero' out of range")
+    stop("argument 'zero' out of range; should have values between ",
+         "1 and ", M, " inclusive")
   if (nasgn[1] != "(Intercept)")
     stop("cannot fit an intercept to a no-intercept model")
 
@@ -422,13 +593,15 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
       Hmatk[zero, ] <- 0
       index <- NULL
       for (kk in 1:ncol(Hmatk))
-        if (all(Hmatk[, kk] == 0)) index <- c(index, kk)
-      if (length(index) == ncol(Hmatk)) 
+        if (all(Hmatk[, kk] == 0))
+          index <- c(index, kk)
+      if (length(index) == ncol(Hmatk))
         stop("constraint matrix has no columns!")
       if (!is.null(index))
         Hmatk <- Hmatk[, -index, drop = FALSE]
-      constraints[[nasgn[ii]]] <- Hmatk 
-    }
+      constraints[[nasgn[ii]]] <- Hmatk
+    }  # for ii
+
   constraints
 }
 
@@ -451,7 +624,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
   if (is.null(constraints)) {
     constraints <- vector("list", length(nasgn))
-    for (ii in 1:length(nasgn))
+    for (ii in seq_along(nasgn))
       constraints[[ii]] <- diag(M)
     names(constraints) <- nasgn
   }
@@ -477,17 +650,17 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
         stop("'constraints[[", ii, "]]' is not a matrix")
     }
 
-  if (is.null(names(constraints))) 
-    names(constraints) <- rep(nasgn, length.out = lenconstraints) 
+  if (is.null(names(constraints)))
+    names(constraints) <- rep_len(nasgn, lenconstraints)
 
   temp <- vector("list", length(nasgn))
   names(temp) <- nasgn
-  for (ii in 1:length(nasgn))
+  for (ii in seq_along(nasgn))
     temp[[nasgn[ii]]] <-
       if (is.null(constraints[[nasgn[ii]]])) diag(M) else
              eval(constraints[[nasgn[ii]]])
 
-  for (ii in 1:length(asgn)) {
+  for (ii in seq_along(asgn)) {
     if (!is.matrix(temp[[ii]])) {
       stop("not a constraint matrix")
     }
@@ -500,7 +673,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
   constraints <- temp
   Hlist <- vector("list", ncol(x))
-  for (ii in 1:length(asgn)) {
+  for (ii in seq_along(asgn)) {
     cols <- asgn[[ii]]
     ictr <- 0
     for (jay in cols) {
@@ -512,7 +685,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
             } else {
               constraints[[ii]]
             }
-      Hlist[[jay]] <- cm 
+      Hlist[[jay]] <- cm
     }
   }
   names(Hlist) <- dimnames(x)[[2]]
@@ -541,6 +714,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
  trivial.constraints <- function(Hlist, target = diag(M)) {
 
 
@@ -551,13 +725,13 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
     Hlist <- list(Hlist)
   M <- dim(Hlist[[1]])[1]
 
-  if (!is.matrix(target)) 
+  if (!is.matrix(target))
     stop("target is not a matrix")
-  dimtar <- dim(target) 
+  dimtar <- dim(target)
 
-  trivc <- rep(1, length(Hlist))
+  trivc <- rep_len(1, length(Hlist))
   names(trivc) <- names(Hlist)
-  for (ii in 1:length(Hlist)) {
+  for (ii in seq_along(Hlist)) {
     d <- dim(Hlist[[ii]])
     if (d[1] != dimtar[1]) trivc[ii] <- 0
     if (d[2] != dimtar[2]) trivc[ii] <- 0
@@ -574,6 +748,9 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
+
+
  add.constraints <- function(constraints, new.constraints,
                              overwrite = FALSE, check = FALSE) {
 
@@ -582,7 +759,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
   if (empty.list(constraints))
     if (is.list(new.constraints))
-      return(new.constraints) else 
+      return(new.constraints) else
       return(list())  # Both NULL probably
 
   constraints <- as.list(constraints)
@@ -605,7 +782,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
                "of the constraints")
         if (overwrite)
           constraints[[ii]] <- new.constraints[[ii]]
-      } else 
+      } else
         constraints[[ii]] <- new.constraints[[ii]]
     }
   } else {
@@ -631,15 +808,16 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
  iam <- function(j, k, M,  # hbw = M,
                  both = FALSE, diag = TRUE) {
 
 
-  jay <- j 
+  jay <- j
   kay <- k
 
   if (M == 1)
-    if (!diag) stop("cannot handle this") 
+    if (!diag) stop("cannot handle this")
 
   if (M == 1)
     if (both) return(list(row.index = 1, col.index = 1)) else return(1)
@@ -649,7 +827,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   i2 <- lapply(i2, seq)
   i2 <- unlist(i2)
 
-  i1 <- matrix(1:M, M, M) 
+  i1 <- matrix(1:M, M, M)
   i1 <- if (diag) c(i1[row(i1) >= col(i1)]) else
                   c(i1[row(i1) >  col(i1)])
 
@@ -660,7 +838,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
       stop("range error in j or k")
     both <- (i1 == jay & i2 == kay) |
             (i1 == kay & i2 == jay)
-    (1:length(i2))[both]
+    (seq_along(i2))[both]
   }
 }
 
@@ -668,6 +846,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
  dimm <- function(M, hbw = M) {
 
   if (!is.numeric(hbw))
@@ -675,7 +854,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
   if (hbw > M || hbw < 1)
     stop("range error in argument 'hbw'")
-  hbw * (2 * M - hbw +1) / 2 
+  hbw * (2 * M - hbw +1) / 2
 }
 
 
@@ -698,16 +877,16 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   dimm <- ncol(m)
   index <- iam(NA, NA, M = M, both = TRUE, diag = TRUE)
   if (dimm > length(index$row.index))
-    stop("bad value for 'M'; it is too small") 
+    stop("bad value for 'M'; it is too small")
   if (dimm < M) {
-    stop("bad value for 'M'; it is too big") 
+    stop("bad value for 'M'; it is too big")
   }
 
-  fred <- .C("m2a", as.double(t(m)), ans=double(M*M*n),
+  fred <- .C("m2accc", as.double(t(m)), ans=double(M*M*n),
       as.integer(dimm),
-      as.integer(index$row-1),  
-      as.integer(index$col-1),  
-      as.integer(n),  as.integer(M),  
+      as.integer(index$row-1),
+      as.integer(index$col-1),
+      as.integer(n),  as.integer(M),
       as.integer(as.numeric(upper)), NAOK = TRUE)
   dim(fred$ans) <- c(M, M, n)
   alpn <- NULL
@@ -717,6 +896,9 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
+
+
  a2m <- function(a, hbw = M) {
 
 
@@ -732,15 +914,15 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   index <- iam(NA, NA, M, both = TRUE, diag = TRUE)
 
 
-  fred <- .C("a2m",
+  fred <- .C("a2mccc",
              as.double(a), m = double(dimm.value * n),
       as.integer(dimm.value),
-      as.integer(index$row-1),  as.integer(index$col-1),  
+      as.integer(index$row-1),  as.integer(index$col-1),
       as.integer(n),  as.integer(M), NAOK = TRUE)
   dim(fred$m) <- c(dimm.value,n)
   fred$m <- t(fred$m)
 
-  if (hbw != M) 
+  if (hbw != M)
     attr(fred$m, "hbw") <- hbw
   if (length(lpn <- dimnames(a)[[1]]) != 0)
     attr(fred$m, "predictors.names") <- lpn
@@ -749,13 +931,16 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
+
+
  vindex <- function(M, row.arg = FALSE, col.arg = FALSE,
                     length.arg = M * (M + 1) / 2) {
 
 
 
   if ((row.arg + col.arg) != 1)
-    stop("only one of row and col must be TRUE") 
+    stop("only one of row and col must be TRUE")
   if (M == 1) {
     ans <- 1
   } else {
@@ -763,19 +948,20 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
       i1 <- matrix(1:M, M, M)
       ans <- c(i1[row(i1) + col(i1) <= (M + 1)])
     } else {
-      i1 <- matrix(1:M, M, M) 
+      i1 <- matrix(1:M, M, M)
       ans <- c(i1[row(i1) >= col(i1)])
     }
   }
   if (length.arg > length(ans))
     stop("argument 'length.arg' too big")
-  rep(ans, length.out = length.arg) 
+  rep_len(ans, length.arg)
 }
 
 
 
 
 
+
  wweights <- function(object, matrix.arg = TRUE, deriv.arg = FALSE,
                       ignore.slot = FALSE, checkwz = TRUE) {
 
@@ -785,8 +971,8 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
-  if (length(wz <- object@weights) && !ignore.slot && !deriv.arg) { 
-    return(wz) 
+  if (length(wz <- object@weights) && !ignore.slot && !deriv.arg) {
+    return(wz)
   }
 
   M <- object@misc$M  # Done below
@@ -802,16 +988,16 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   if (any(slotNames(object) == "predictors"))
     eta <- object@predictors
   mt <- terms(object)  # object@terms$terms; 20030811
-  Hlist <- constraints <- object@constraints 
+  Hlist <- constraints <- object@constraints
   new.coeffs <- object@coefficients
   if (any(slotNames(object) == "iter"))
     iter <- object@iter
 
-  w <- rep(1, n)
+  w <- rep_len(1, n)
   if (any(slotNames(object) == "prior.weights"))
     w <- object@prior.weights
   if (!length(w))
-    w <- rep(1, n)
+    w <- rep_len(1, n)
 
   x <- object@x
   if (!length(x))
@@ -846,12 +1032,12 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   if (any(slotNames(object) == "control"))
     for (ii in names(object@control)) {
       assign(ii, object@control[[ii]])
-    } 
+    }
 
   if (length(object@misc))
     for (ii in names(object@misc)) {
-      assign(ii, object@misc[[ii]]) 
-    } 
+      assign(ii, object@misc[[ii]])
+    }
 
   if (any(slotNames(object) == "family")) {
     expr <- object@family@deriv
@@ -861,28 +1047,30 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
       wz <- eval(expr)
 
 
-      if (M > 1) 
+      if (M > 1)
         dimnames(wz) <- list(dimnames(wz)[[1]], NULL)  # Remove colnames
-      wz <- if (matrix.arg) as.matrix(wz) else c(wz) 
+      wz <- if (matrix.arg) as.matrix(wz) else c(wz)
     }
     if (deriv.arg) list(deriv = deriv.mu, weights = wz) else wz
   } else {
-    NULL 
+    NULL
   }
 }
 
 
 
 
+
+
  pweights <- function(object, ...) {
   ans <- object@prior.weights
   if (length(ans)) {
-    ans 
+    ans
   } else {
     temp <- object@y
-    ans <- rep(1, nrow(temp))  # Assumed all equal and unity.
+    ans <- rep_len(1, nrow(temp))  # Assumed all equal and unity.
     names(ans) <- dimnames(temp)[[1]]
-    ans 
+    ans
   }
 }
 
@@ -890,6 +1078,9 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
+
+
+
 procVec <- function(vec, yn, Default) {
 
 
@@ -899,7 +1090,7 @@ procVec <- function(vec, yn, Default) {
 
 
 
-  if (any(is.na(vec)))
+  if (anyNA(vec))
     stop("vec cannot contain any NAs")
   L <- length(vec)
   nvec <- names(vec)  # vec[""] undefined
@@ -911,18 +1102,18 @@ procVec <- function(vec, yn, Default) {
     default <- vec
   }
 
-  answer <- rep(default, length.out = length(yn))
+  answer <- rep_len(default, length(yn))
   names(answer) <- yn
   if (named) {
     nvec2 <- nvec[nvec != ""]
     if (length(nvec2)) {
       if (any(!is.element(nvec2, yn)))
           stop("some names given which are superfluous")
-      answer <- rep(as.numeric(NA), length.out = length(yn))
+      answer <- rep_len(NA_real_, length(yn))
       names(answer) <- yn
       answer[nvec2] <- vec[nvec2]
       answer[is.na(answer)] <-
-        rep(default, length.out <- sum(is.na(answer)))
+        rep_len(default, sum(is.na(answer)))
     }
   }
 
@@ -938,6 +1129,8 @@ if (FALSE) {
 
 
 
+
+
  weightsvglm <- function(object, type = c("prior", "working"),
                         matrix.arg = TRUE, ignore.slot = FALSE,
                         deriv.arg = FALSE, ...) {
@@ -968,6 +1161,7 @@ if (FALSE) {
 }
 
 
+
 if (!isGeneric("weights"))
     setGeneric("weights", function(object, ...)
   standardGeneric("weights"))
@@ -993,10 +1187,9 @@ setMethod("weights", "vglm",
 
 
 
-
-qnupdate <- function(w, wzold, dderiv, deta, M, keeppd = TRUE, 
-                    trace = FALSE, reset = FALSE,
-                    effpos=.Machine$double.eps^0.75) {
+qnupdate <- function(w, wzold, dderiv, deta, M, keeppd = TRUE,
+                     trace = FALSE, reset = FALSE,
+                     effpos=.Machine$double.eps^0.75) {
 
 
   if (M == 1) {
@@ -1005,13 +1198,13 @@ qnupdate <- function(w, wzold, dderiv, deta, M, keeppd = TRUE,
   }
   Bs <- mux22(t(wzold), deta, M = M,
               upper = FALSE, as.matrix = TRUE)  # n x M
-  sBs <- c( (deta * Bs) %*% rep(1, M) )  # should have positive values
-  sy <- c( (dderiv * deta) %*% rep(1, M) )
+  sBs <- c( (deta * Bs) %*% rep_len(1, M) )  # should have positive values
+  sy <- c( (dderiv * deta) %*% rep_len(1, M) )
   wznew <- wzold
   index <- iam(NA, NA, M = M, both = TRUE)
-  index$row.index <- rep(index$row.index, len=ncol(wzold))
-  index$col.index <- rep(index$col.index, len=ncol(wzold))
-  updateThese <- if (keeppd) (sy > effpos) else rep(TRUE, len=length(sy))
+  index$row.index <- rep_len(index$row.index, ncol(wzold))
+  index$col.index <- rep_len(index$col.index, ncol(wzold))
+  updateThese <- if (keeppd) (sy > effpos) else rep_len(TRUE, length(sy))
   if (!keeppd || any(updateThese)) {
     wznew[updateThese,] <- wznew[updateThese,] -
         Bs[updateThese,index$row] *
@@ -1050,9 +1243,9 @@ mbesselI0 <- function(x, deriv.arg = 0) {
   if (FALSE) {
     }
 
-    ans <- matrix(as.numeric(NA), nrow = nn, ncol = deriv.arg+1)
+    ans <- matrix(NA_real_, nrow = nn, ncol = deriv.arg+1)
     ans[, 1] <- besselI(x, nu = 0)
-    if (deriv.arg>=1) ans[,2] <- besselI(x, nu = 1) 
+    if (deriv.arg>=1) ans[,2] <- besselI(x, nu = 1)
     if (deriv.arg>=2) ans[,3] <- ans[,1] - ans[,2] / x
     ans
 }
@@ -1064,11 +1257,11 @@ VGAM.matrix.norm <- function(A, power = 2, suppressWarning = FALSE) {
     warning("norms should be calculated for square matrices; ",
             "'A' is not square")
   if (power == "F") {
-    sqrt(sum(A^2)) 
+    sqrt(sum(A^2))
   } else if (power == 1) {
     max(colSums(abs(A)))
   } else if (power == 2) {
-    sqrt(max(eigen(t(A) %*% A)$value))
+    sqrt(max(eigen(t(A) %*% A, symmetric = TRUE)$value))
   } else if (!is.finite(power)) {
     max(colSums(abs(A)))
   } else {
@@ -1111,7 +1304,7 @@ existsinVGAMenv <- function(varnames, prefix = "") {
 
 assign2VGAMenv <- function(varnames, mylist, prefix = "") {
   evarnames <- paste(prefix, varnames, sep = "")
-  for (ii in 1:length(varnames)) {
+  for (ii in seq_along(varnames)) {
     assign(evarnames[ii], mylist[[(varnames[ii])]],
            envir = VGAMenv)
   }
@@ -1129,7 +1322,7 @@ getfromVGAMenv <- function(varname, prefix = "") {
   get(varname, envir = VGAMenv)
 }
 
- 
+
 
 lerch <- function(x, s, v, tolerance = 1.0e-10, iter = 100) {
   if (!is.Numeric(x) || !is.Numeric(s) || !is.Numeric(v))
@@ -1144,9 +1337,9 @@ lerch <- function(x, s, v, tolerance = 1.0e-10, iter = 100) {
     stop("bad input for argument 'iter'")
 
   L <- max(length(x), length(s), length(v))
-  x <- rep(x, length.out = L);
-  s <- rep(s, length.out = L);
-  v <- rep(v, length.out = L);
+  x <- rep_len(x, L)
+  s <- rep_len(s, L)
+  v <- rep_len(v, L)
   xok <- abs(x) < 1 & !(v <= 0 & v == round(v))
   x[!xok] <- 0  # Fix this later
 
@@ -1170,10 +1363,46 @@ negzero.expression.VGAM <- expression({
 
 
 
+
+
+
+  if (length(dotzero) == 1 && (dotzero == "" || is.na(dotzero)))
+    dotzero <- NULL
+
+
+
+  if (is.character(dotzero)) {
+
+
+
+
+  which.numeric.all <- NULL
+  for (ii in seq_along(dotzero)) {
+    which.ones <-
+        grep(dotzero[ii], predictors.names, fixed = TRUE)
+    if (length(which.ones)) {
+      which.numeric.all <- c(which.numeric.all, which.ones)
+    } else {
+      warning("some values of argument 'zero' are unmatched. Ignoring them")
+    }
+  }
+  which.numeric <- unique(sort(which.numeric.all))
+
+  if (!length(which.numeric)) {
+    warning("No values of argument 'zero' were matched.")
+    which.numeric <- NULL
+  } else if (length(which.numeric.all) > length(which.numeric)) {
+    warning("There were redundant values of argument 'zero'.")
+  }
+
+    dotzero <- which.numeric
+  }
+
+
+
   posdotzero <-  dotzero[dotzero > 0]
   negdotzero <-  dotzero[dotzero < 0]
 
-  bigUniqInt <- 1080
   zneg.index <- if (length(negdotzero)) {
 
     if (!is.Numeric(-negdotzero, positive = TRUE,
@@ -1181,6 +1410,7 @@ negzero.expression.VGAM <- expression({
         max(-negdotzero) > M1)
         stop("bad input for argument 'zero'")
 
+    bigUniqInt <- 1080
     zneg.index <- rep(0:bigUniqInt, rep(length(negdotzero),
                       1 + bigUniqInt)) * M1 + abs(negdotzero)
     sort(intersect(zneg.index, 1:M))
@@ -1189,8 +1419,9 @@ negzero.expression.VGAM <- expression({
   }
 
   zpos.index <- if (length(posdotzero)) posdotzero else NULL
-  z.Index <- if (!length(dotzero)) NULL else
-                   unique(sort(c(zneg.index, zpos.index)))
+  z.Index <- if (!length(dotzero))
+               NULL else
+               unique(sort(c(zneg.index, zpos.index)))
 
   constraints <- cm.zero.VGAM(constraints, x = x, z.Index, M = M)
 })
@@ -1210,8 +1441,20 @@ is.empty.list <- function(mylist) {
 
 
 
-interleave.VGAM <- function(L, M)
-  c(matrix(1:L, nrow = M, byrow = TRUE))
+
+
+
+  interleave.VGAM  <- function(.M, M1, inverse = FALSE) {
+  if (inverse) {
+    NRs <- (.M)/M1
+    if (round(NRs) != NRs)
+      stop("Incompatible number of parameters")
+    c(matrix(1:(.M), nrow = NRs, byrow = TRUE))
+  } else {
+    c(matrix(1:(.M), nrow = M1, byrow = TRUE))
+  }
+}
+
 
 
 
@@ -1231,7 +1474,7 @@ interleave.cmat <- function(cmat1, cmat2) {
       return(cbind(cmat1[, 1], cmat2, cmat1[, -1]))
     } else {  # ncol1 == ncol2 and both are > 1.
       kronecker(cmat1, cbind(1, 0)) +
-      kronecker(cmat2, cbind(0, 1))        
+      kronecker(cmat2, cbind(0, 1))
     }
   }
 }
@@ -1257,7 +1500,7 @@ w.wz.merge <- function(w, wz, n, M, ndepy,
   if (intercept.only)
     warning("yettodo: support intercept.only == TRUE")
 
-  if (ncol(as.matrix(w)) > ndepy)
+  if (NCOL(w) > ndepy)
     stop("number of columns of 'w' exceeds number of responses")
 
   w  <- matrix(w, n, ndepy)
@@ -1380,7 +1623,7 @@ w.y.check <- function(w, y,
                             sep = ""))
     y <- matrix(y, n.lm, Ncol.max.y, dimnames = list(rn.y, cn.y))
   }
-       
+
   list(w = if (out.wy) w else NULL,
        y = if (out.wy) y else NULL)
 }
@@ -1389,7 +1632,8 @@ w.y.check <- function(w, y,
 
 
 
-arwz2wz <- function(arwz, M = 1, M1 = 1) {
+arwz2wz <- function(arwz, M = 1, M1 = 1, rm.trailing.cols = TRUE,
+                    full.arg = FALSE) {
 
 
 
@@ -1404,7 +1648,7 @@ arwz2wz <- function(arwz, M = 1, M1 = 1) {
     return(arwz)
   }
 
-  wz <- matrix(0.0, nrow = n, ncol = sum(M:(M-M1+1)))
+  wz <- matrix(0.0, n, if (full.arg) M*(M+1)/2 else sum(M:(M-M1+1)))
   ind1 <- iam(NA, NA, M = M1, both = TRUE, diag = TRUE)
   len.ind1 <- dim.val # length(ind1$col.index)
 
@@ -1416,12 +1660,13 @@ arwz2wz <- function(arwz, M = 1, M1 = 1) {
     }
   }
 
-  colind <- ncol(wz)
-  while (all(wz[, colind] == 0))
-    colind <- colind - 1
-
-  if (colind < ncol(wz))
-    wz <- wz[, 1:colind, drop = FALSE]
+  if (rm.trailing.cols && !full.arg) {
+    colind <- ncol(wz)
+    while (all(wz[, colind] == 0))
+      colind <- colind - 1
+    if (colind < ncol(wz))
+      wz <- wz[, 1:colind, drop = FALSE]
+  }
 
   wz
 }
@@ -1502,3 +1747,39 @@ setMethod("familyname", "vlm",
 
 
 
+bisection.basic <- function(f, a, b, tol = 1e-9, nmax = 500, ...) {
+
+
+
+  if (!all(sign(f(a, ...)) * sign(f(b, ...)) <= 0))
+    stop("roots do not exist between 'a' and 'b'")
+
+  N <- 1
+  while (N <= nmax) {
+    mid <- (a + b) / 2
+    save.f <- f(mid, ...)
+    if (all(save.f == 0 | (b - a)/2 < tol)) {
+      return(mid)
+    }
+    N <- N + 1
+    vecTF <- sign(save.f) == sign(f(a, ...))
+    a[ vecTF] <- mid[ vecTF]
+    b[!vecTF] <- mid[!vecTF]
+  }
+
+  warning("did not coverge. Returning final root")
+  mid
+}
+
+
+
+
+
+retain.col <- function(mat, coln
+                      ) {
+  if (is.matrix(mat))  # && exclude
+    mat[, -coln] <- 0
+  mat
+}
+
+
