@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -28,7 +28,7 @@
             "Link:     ", namesof("mu", link, earg = earg), "\n",
             "Variance: mu"),
   initialize = eval(substitute(expression({
-    if (any(is.na(y)))
+    if (anyNA(y))
       stop("NAs are not allowed in the response")
 
 
@@ -44,16 +44,16 @@
         temp <- y[, 2]
         extra$uncensored <- ifelse(temp == 1, TRUE, FALSE)
         extra$rightcensored <- ifelse(temp == 0, TRUE, FALSE)
-        extra$leftcensored <- rep(FALSE, len = n)
-        extra$interval <- rep(FALSE, len = n)
+        extra$leftcensored <- rep_len(FALSE, n)
+        extra$interval <- rep_len(FALSE, n)
         init.mu <- pmax(y[, 1], 1/8)
     } else
     if (centype == "left") {
         temp <- y[, 2]
         extra$uncensored <- ifelse(temp == 1, TRUE, FALSE)
-        extra$rightcensored <- rep(FALSE, len = n)
+        extra$rightcensored <- rep_len(FALSE, n)
         extra$leftcensored <- ifelse(temp == 0, TRUE, FALSE)
-        extra$interval <- rep(FALSE, len = n)
+        extra$interval <- rep_len(FALSE, n)
         init.mu <- pmax(y[, 1], 1/8)
     } else
     if (centype == "interval" ||
@@ -79,7 +79,7 @@
       stop("response have to be in a class of SurvS4")
 
       if (length( .imu )) init.mu <- 0 * y[, 1] + .imu
-  
+
       predictors.names <-
         namesof("mu", .link, earg = .earg, short = TRUE)
 
@@ -119,12 +119,17 @@
     }
   },
   vfamily = "cens.poisson",
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    lambda <- eta2theta(eta, link = .link , earg = .earg )
+    okay1 <- all(is.finite(lambda)) && all(0 < lambda)
+    okay1
+  }, list(  .link = link, .earg = earg ))),
   deriv = eval(substitute(expression({
     cen0 <- extra$uncensored
     cenL <- extra$leftcensored
     cenU <- extra$rightcensored
     cenI <- extra$intervalcensored
-      lambda <- eta2theta(eta, link = .link, earg = .earg)
+      lambda <- eta2theta(eta, link = .link , earg = .earg )
 
       dl.dlambda <- (y[, 1] - lambda)/lambda   # uncensored
 
@@ -188,7 +193,7 @@
 
 
 if (FALSE)
- cens.exponential <- 
+ cens.exponential <-
  ecens.exponential <- function(link = "loge", location = 0) {
   if (!is.Numeric(location, length.arg = 1))
     stop("bad input for 'location'")
@@ -230,22 +235,22 @@ if (FALSE)
       temp <- y[, 2]
       extra$uncensored <- ifelse(temp == 1, TRUE, FALSE)
       extra$rightcensored <- ifelse(temp == 0, TRUE, FALSE)
-      extra$leftcensored <- rep(FALSE, len = n)
-      extra$interval <- rep(FALSE, len = n)
+      extra$leftcensored <- rep_len(FALSE, n)
+      extra$interval <- rep_len(FALSE, n)
     } else
     if (type == "left") {
       temp <- y[, 2]
       extra$uncensored <- ifelse(temp == 1, TRUE, FALSE)
-      extra$rightcensored <- rep(FALSE, len = n)
+      extra$rightcensored <- rep_len(FALSE, n)
       extra$leftcensored <- ifelse(temp == 0, TRUE, FALSE)
-      extra$interval <- rep(FALSE, len = n)
+      extra$interval <- rep_len(FALSE, n)
     } else
     if (type == "counting") {
       stop("type == 'counting' not recognized")
       extra$uncensored <- rep(temp == 1, TRUE, FALSE)
-      extra$interval <- rep(FALSE, len = n)
-      extra$leftcensored <- rep(FALSE, len = n)
-      extra$rightcensored <- rep(FALSE, len = n)
+      extra$interval <- rep_len(FALSE, n)
+      extra$leftcensored <- rep_len(FALSE, n)
+      extra$rightcensored <- rep_len(FALSE, n)
       extra$counting <- ifelse(temp == 0, TRUE, FALSE)
     } else
     if (type == "interval") {
@@ -286,6 +291,11 @@ if (FALSE)
     exp(-rate[cenI]*(y[cenI, 1]-extra$location))))
   }, list( .link = link ))),
   vfamily = c("ecens.exponential"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    rate <- 1 / (mu - extra$location)
+    okay1 <- all(is.finite(rate)) && all(0 < rate)
+    okay1
+  }, list(  .link = link ))),
   deriv = eval(substitute(expression({
     rate <- 1 / (mu - extra$location)
     cen0 <- extra$uncensored
@@ -336,8 +346,7 @@ if (FALSE)
 
  cennormal <-
  cens.normal <- function(lmu = "identitylink", lsd = "loge",
-                         imethod = 1, zero = 2) {
-
+                         imethod = 1, zero = "sd") {
 
 
   lmu <- as.list(substitute(lmu))
@@ -349,7 +358,6 @@ if (FALSE)
   lsd <- attr(esd, "function.name")
 
 
-
   if (!is.Numeric(imethod, length.arg = 1,
                   integer.valued = TRUE, positive = TRUE) ||
     imethod > 2)
@@ -362,8 +370,19 @@ if (FALSE)
                           namesof("sd", lsd, tag = TRUE), "\n",
             "Conditional variance: sd^2"),
   constraints = eval(substitute(expression({
-    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M)
+    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M,
+                                predictors.names = predictors.names,
+                                M1 = 2)
   }), list( .zero = zero ))),
+  infos = eval(substitute(function(...) {
+    list(M1 = 2,
+         Q1 = 1,
+         zero = .zero ,
+         multiple.responses = FALSE,
+         parameters.names = c("mu", "sd"),
+         expected = TRUE )
+  }, list( .zero = zero ))),
+
   initialize = eval(substitute(expression({
 
     temp5 <-
@@ -376,15 +395,15 @@ if (FALSE)
 
 
     if (!length(extra$leftcensored))
-      extra$leftcensored <- rep(FALSE, len = n)
+      extra$leftcensored <- rep_len(FALSE, n)
     if (!length(extra$rightcensored))
-      extra$rightcensored <- rep(FALSE, len = n)
+      extra$rightcensored <- rep_len(FALSE, n)
     if (any(extra$rightcensored & extra$leftcensored))
         stop("some observations are both right and left censored!")
 
     predictors.names <-
-      c(namesof("mu", .lmu , earg =.emu, tag = FALSE),
-        namesof("sd", .lsd, earg =.esd, tag = FALSE))
+      c(namesof("mu", .lmu , earg = .emu , tag = FALSE),
+        namesof("sd", .lsd , earg = .esd , tag = FALSE))
 
     if (!length(etastart)) {
       anyc <- extra$leftcensored | extra$rightcensored
@@ -393,7 +412,7 @@ if (FALSE)
                       y = y[!i11], w = w[!i11])
       sd.y.est <- sqrt(sum(w[!i11] * junk$resid^2) / junk$df.residual)
       etastart <- cbind(mu = y,
-                        rep(theta2eta(sd.y.est, .lsd), length = n))
+                        rep_len(theta2eta(sd.y.est, .lsd), n))
       if (any(anyc))
         etastart[anyc, 1] <- x[anyc, , drop = FALSE] %*% junk$coeff
     }
@@ -404,9 +423,9 @@ if (FALSE)
     eta2theta(eta[, 1], .lmu , earg = .emu )
   }, list( .lmu = lmu, .emu = emu ))),
   last = eval(substitute(expression({
-    misc$link <-    c("mu" = .lmu , "sd" = .lsd)
+    misc$link <-    c("mu" = .lmu , "sd" = .lsd )
 
-    misc$earg <- list("mu" = .emu ,"sd" = .esd )
+    misc$earg <- list("mu" = .emu , "sd" = .esd )
 
     misc$expected <- TRUE
     misc$multipleResponses <- FALSE
@@ -419,7 +438,7 @@ if (FALSE)
     cen0 <- !cenL & !cenU   # uncensored obsns
 
     mum <- eta2theta(eta[, 1], .lmu , earg = .emu )
-    sdv <- eta2theta(eta[, 2], .lsd, earg = .esd )
+    sdv <- eta2theta(eta[, 2], .lsd , earg = .esd )
 
     Lower <- ifelse(cenL, y, -Inf)
     Upper <- ifelse(cenU, y,  Inf)
@@ -428,10 +447,20 @@ if (FALSE)
     ell3 <- log1p(-pnorm(( Upper[cenU] -  mum[cenU]) / sdv[cenU]))
     if (residuals) stop("loglikelihood residuals not ",
                         "implemented yet") else
-    sum(w[cen0] * ell1) + sum(w[cenL] * ell2) + sum(w[cenU] * ell3)
+    sum(w[cen0] * ell1) +
+    sum(w[cenL] * ell2) +
+    sum(w[cenU] * ell3)
   }, list( .lmu = lmu, .lsd = lsd,
            .emu = emu, .esd = esd ))),
   vfamily = c("cens.normal"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    mum <- eta2theta(eta[, 1], .lmu )
+    sdv <- eta2theta(eta[, 2], .lsd )
+    okay1 <- all(is.finite(mum)) &&
+             all(is.finite(sdv)) && all(0 < sdv)
+    okay1
+  }, list( .lmu = lmu, .lsd = lsd,
+           .emu = emu, .esd = esd ))),
   deriv = eval(substitute(expression({
     cenL <- extra$leftcensored
     cenU <- extra$rightcensored
@@ -439,14 +468,14 @@ if (FALSE)
     Lower <- ifelse(cenL, y, -Inf)
     Upper <- ifelse(cenU, y,  Inf)
 
-    mum <- eta2theta(eta[, 1], .lmu)
-    sdv <- eta2theta(eta[, 2], .lsd)
+    mum <- eta2theta(eta[, 1], .lmu )
+    sdv <- eta2theta(eta[, 2], .lsd )
 
     dl.dmu <- (y-mum) / sdv^2
     dl.dsd <- (((y-mum)/sdv)^2 - 1) / sdv
 
     dmu.deta <- dtheta.deta(mum, .lmu , earg = .emu )
-    dsd.deta <- dtheta.deta(sdv, .lsd, earg = .esd )
+    dsd.deta <- dtheta.deta(sdv, .lsd , earg = .esd )
 
     if (any(cenL)) {
       mumL <- mum - Lower
@@ -477,24 +506,24 @@ if (FALSE)
     A3 <- 1 - pnorm((Upper - mum) / sdv)  # Upper
     A2 <- 1 - A1 - A3                     # Middle; uncensored
     wz <- matrix(0, n, 3)
-    wz[,iam(1, 1,M)] <- A2 * 1 / sdv^2  # ed2l.dmu2
-    wz[,iam(2, 2,M)] <- A2 * 2 / sdv^2  # ed2l.dsd2
+    wz[, iam(1, 1,M)] <- A2 * 1 / sdv^2  # ed2l.dmu2
+    wz[, iam(2, 2,M)] <- A2 * 2 / sdv^2  # ed2l.dsd2
     mumL <- mum - Lower
     temp21L <- mumL / sdv
     PhiL <- pnorm(temp21L)
     phiL <- dnorm(temp21L)
-    temp31L <- ((1-PhiL) * sdv)^2 
+    temp31L <- ((1-PhiL) * sdv)^2
     wz.cenL11 <- phiL * (phiL - (1-PhiL)*temp21L) / temp31L
     wz.cenL22 <- mumL * phiL * ((1-PhiL) * (2 - temp21L^2) +
-                mumL * phiL / sdv) / (sdv * temp31L)
+                 mumL * phiL / sdv) / (sdv * temp31L)
     wz.cenL12 <- phiL * ((1-PhiL)*(temp21L^2 - 1) -
-                temp21L*phiL) / temp31L
+                 temp21L*phiL) / temp31L
     wz.cenL11[!is.finite(wz.cenL11)] <- 0
     wz.cenL22[!is.finite(wz.cenL22)] <- 0
     wz.cenL12[!is.finite(wz.cenL12)] <- 0
-    wz[,iam(1, 1,M)] <- wz[,iam(1, 1,M)] + A1 * wz.cenL11
-    wz[,iam(2, 2,M)] <- wz[,iam(2, 2,M)] + A1 * wz.cenL22
-    wz[,iam(1, 2,M)] <- A1 * wz.cenL12
+    wz[, iam(1, 1, M)] <- wz[, iam(1, 1, M)] + A1 * wz.cenL11
+    wz[, iam(2, 2, M)] <- wz[, iam(2, 2, M)] + A1 * wz.cenL22
+    wz[, iam(1, 2, M)] <- A1 * wz.cenL12
     mumU <- Upper - mum    # often Inf
     temp21U <- mumU / sdv    # often Inf
     PhiU <- pnorm(temp21U)  # often 1
@@ -505,16 +534,16 @@ if (FALSE)
     tmp9 <- (1-PhiU) * (2 - temp21U^2)
     wzcenU22 <- mumU * phiU * (tmp9 + mumU * phiU / sdv) / (sdv * temp31U)
     wzcenU12 <- -phiU * ((1-PhiU)*(temp21U^2 - 1) -
-                temp21U*phiU) / temp31U
+                 temp21U*phiU) / temp31U
     wzcenU11[!is.finite(wzcenU11)] <- 0  # Needed when Upper==Inf
     wzcenU22[!is.finite(wzcenU22)] <- 0  # Needed when Upper==Inf
     wzcenU12[!is.finite(wzcenU12)] <- 0  # Needed when Upper==Inf
-    wz[,iam(1, 1,M)] <- wz[,iam(1, 1,M)] + A3 * wzcenU11
-    wz[,iam(2, 2,M)] <- wz[,iam(2, 2,M)] + A3 * wzcenU22
-    wz[,iam(1, 2,M)] <- wz[,iam(1, 2,M)] + A3 * wzcenU12
-    wz[,iam(1, 1,M)] <- wz[,iam(1, 1,M)] * dmu.deta^2
-    wz[,iam(2, 2,M)] <- wz[,iam(2, 2,M)] * dsd.deta^2
-    wz[,iam(1, 2,M)] <- wz[,iam(1, 2,M)] * dmu.deta * dsd.deta
+    wz[, iam(1, 1, M)] <- wz[, iam(1, 1, M)] + A3 * wzcenU11
+    wz[, iam(2, 2, M)] <- wz[, iam(2, 2, M)] + A3 * wzcenU22
+    wz[, iam(1, 2, M)] <- wz[, iam(1, 2, M)] + A3 * wzcenU12
+    wz[, iam(1, 1, M)] <- wz[, iam(1, 1, M)] * dmu.deta^2
+    wz[, iam(2, 2, M)] <- wz[, iam(2, 2, M)] * dsd.deta^2
+    wz[, iam(1, 2, M)] <- wz[, iam(1, 2, M)] * dmu.deta * dsd.deta
     c(w) * wz
   }), list( .lmu = lmu, .lsd = lsd ))))
 }
@@ -541,14 +570,14 @@ if (FALSE)
             namesof("scale", lscale, earg = escale ), "\n", "\n",
             "Mean:    scale * sqrt(pi / 2)"),
   initialize = eval(substitute(expression({
-    if (ncol(cbind(y)) != 1)
+    if (NCOL(y) != 1)
       stop("response must be a vector or a one-column matrix")
 
     if (length(extra$leftcensored))
       stop("cannot handle left-censored data")
 
     if (!length(extra$rightcensored))
-      extra$rightcensored <- rep(FALSE, len = n)
+      extra$rightcensored <- rep_len(FALSE, n)
 
     predictors.names <-
       namesof("scale", .lscale , earg = .escale , tag = FALSE)
@@ -563,9 +592,9 @@ if (FALSE)
     Scale * sqrt(pi/2)
   }, list( .lscale = lscale, .escale = escale ))),
   last = eval(substitute(expression({
-    misc$link <-    c("scale" = .lscale)
+    misc$link <-    c("scale" = .lscale )
 
-    misc$earg <- list("scale" = .escale)
+    misc$earg <- list("scale" = .escale )
 
     misc$oim <- .oim
   }), list( .lscale = lscale, .escale = escale,
@@ -582,9 +611,13 @@ if (FALSE)
       sum(w[cen0] * (log(y[cen0]) - 2*log(Scale[cen0]) -
                      0.5*(y[cen0]/Scale[cen0])^2)) -
       sum(w[cenU] * (y[cenU]/Scale[cenU])^2) * 0.5
-  }, list( .lscale = lscale,
-           .escale = escale ))),
+  }, list( .lscale = lscale, .escale = escale ))),
   vfamily = c("cens.rayleigh"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    Scale <- eta2theta(eta, .lscale , earg = .escale )
+    okay1 <- all(is.finite(Scale)) && all(0 < Scale)
+    okay1
+  }, list( .lscale = lscale, .escale = escale ))),
   deriv = eval(substitute(expression({
     cen0 <- !extra$rightcensored   # uncensored obsns
     cenU <- extra$rightcensored
@@ -631,7 +664,7 @@ if (FALSE)
   function(lmean = "loge", lshape = "loge",
            imean = NULL,   ishape = NULL,
            probs.y = c(0.2, 0.5, 0.8),
-           imethod = 1, zero = -2) {
+           imethod = 1, zero = "shape") {
 
 
 
@@ -648,9 +681,6 @@ if (FALSE)
   lmeann <- attr(emeann, "function.name")
 
 
-  if (length(zero) &&
-      !is.Numeric(zero, integer.valued = TRUE))
-    stop("bad input for argument 'zero'")
 
   if (!is.Numeric(imethod, length.arg = 1,
                   integer.valued = TRUE, positive = TRUE) ||
@@ -683,9 +713,9 @@ if (FALSE)
             "Variance: mean^2 * (gamma(1 + 2/shape) / ",
                       "gamma(1 + 1/shape)^2 - 1)"),
  constraints = eval(substitute(expression({
-    dotzero <- .zero
-    M1 <- 2
-    eval(negzero.expression.VGAM)
+    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M,
+                                predictors.names = predictors.names,
+                                M1 = 2)
   }), list( .zero = zero,
             .lmeann = lmeann ))),
 
@@ -694,8 +724,12 @@ if (FALSE)
          Q1 = 1,
          expected = TRUE,
          multipleResponses = TRUE,
+         parameters.names = c("mean", "shape"),
+         lmean  = .lmeann ,
+         lshape = .lshape ,
          zero = .zero )
-  }, list( .zero = zero ))),
+  }, list( .zero = zero,
+           .lmeann = lmeann, .lshape = lshape ))),
 
   initialize = eval(substitute(expression({
 
@@ -727,7 +761,7 @@ if (FALSE)
     predictors.names <-
         c(namesof(mynames1, .lmeann , earg = .emeann , tag = FALSE),
           namesof(mynames2, .lshape , earg = .eshape , tag = FALSE))
-    predictors.names <- predictors.names[interleave.VGAM(M, M = M1)]
+    predictors.names <- predictors.names[interleave.VGAM(M, M1 = M1)]
 
 
     Meann.init <- matrix(if (length( .imeann )) .imeann else 0.5 * colMeans(y),
@@ -753,10 +787,10 @@ if (FALSE)
             Shape.init[, ilocal] <- 1 / fit0$coef["X"]
         }  # ilocal
 
-        etastart <- 
+        etastart <-
           cbind(theta2eta(Meann.init, .lmeann , earg = .emeann ),
                 theta2eta(Shape.init, .lshape , earg = .eshape ))[,
-                interleave.VGAM(M, M = M1)]
+                interleave.VGAM(M, M1 = M1)]
       }
     }
   }), list( .lmeann = lmeann, .lshape = lshape,
@@ -790,10 +824,10 @@ if (FALSE)
 
 
     M1 <- extra$M1
-    avector <- c(rep( .lmeann , length = ncoly),
-                 rep( .lshape , length = ncoly))
-    misc$link <- avector[interleave.VGAM(M, M = M1)]
-    temp.names <- c(mynames1, mynames2)[interleave.VGAM(M, M = M1)]
+    avector <- c(rep_len( .lmeann , ncoly),
+                 rep_len( .lshape , ncoly))
+    misc$link <- avector[interleave.VGAM(M, M1 = M1)]
+    temp.names <- c(mynames1, mynames2)[interleave.VGAM(M, M1 = M1)]
     names(misc$link) <- temp.names
 
     misc$earg <- vector("list", M)
@@ -828,6 +862,14 @@ if (FALSE)
   }, list( .lmeann = lmeann, .lshape = lshape,
            .emeann = emeann, .eshape = eshape ) )),
   vfamily = c("weibull.mean"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    Meann <- eta2theta(eta[, c(TRUE, FALSE)], .lmeann , earg = .emeann )
+    Shape <- eta2theta(eta[, c(FALSE, TRUE)], .lshape , earg = .eshape )
+    okay1 <- all(is.finite(Meann)) && all(0 < Meann) &&
+             all(is.finite(Shape)) && all(0 < Shape)
+    okay1
+  }, list( .lmeann = lmeann, .lshape = lshape,
+           .emeann = emeann, .eshape = eshape ) )),
   deriv = eval(substitute(expression({
     M1 <- 2
     Meann <- eta2theta(eta[, c(TRUE, FALSE)], .lmeann , earg = .emeann )
@@ -841,7 +883,7 @@ if (FALSE)
       CC <- y * gamma(1 + 1/Shape) / Meann
       dl.dmeann <- (CC^Shape - 1) * Shape / Meann  # Agrees
       dl.dshape <- 1/Shape -
-                   (log(y/Meann) + lgamma(1 + 1/Shape)) * (CC^Shape - 1) + 
+                   (log(y/Meann) + lgamma(1 + 1/Shape)) * (CC^Shape - 1) +
                    (BB / Shape) * (CC^Shape - 1)
     }
 
@@ -851,7 +893,7 @@ if (FALSE)
 
     myderiv <- c(w) * cbind(dl.dmeann * dmeann.deta,
                             dl.dshape * dshape.deta)
-    myderiv[, interleave.VGAM(M, M = M1)]
+    myderiv[, interleave.VGAM(M, M1 = M1)]
   }), list( .lmeann = lmeann, .lshape = lshape,
             .emeann = emeann, .eshape = eshape ) )),
   weight = eval(substitute(expression({
@@ -886,7 +928,7 @@ if (FALSE)
            lss = TRUE,
            nrfs = 1,
            probs.y = c(0.2, 0.5, 0.8),
-           imethod = 1, zero = ifelse(lss, -2, -1)) {
+           imethod = 1, zero = "shape") {
 
 
 
@@ -899,9 +941,6 @@ if (FALSE)
   lscale <- attr(escale, "function.name")
 
 
-  if (length(zero) &&
-      !is.Numeric(zero, integer.valued = TRUE))
-    stop("bad input for argument 'zero'")
 
   if (!is.Numeric(imethod, length.arg = 1,
                   integer.valued = TRUE, positive = TRUE) ||
@@ -943,9 +982,9 @@ if (FALSE)
             "Variance: scale^2 * (gamma(1 + 2/shape) - ",
                       "gamma(1 + 1/shape)^2)"),
  constraints = eval(substitute(expression({
-    dotzero <- .zero
-    M1 <- 2
-    eval(negzero.expression.VGAM)
+    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M,
+                                predictors.names = predictors.names,
+                                M1 = 2)
   }), list( .zero = zero,
             .scale.12 = scale.12, .scale.TF = scale.TF, .lss = lss ))),
 
@@ -954,8 +993,17 @@ if (FALSE)
          Q1 = 1,
          expected = TRUE,
          multipleResponses = TRUE,
+         parameters.names = if ( .lss )
+           c("scale", "shape") else
+           c("shape", "scale"),
+         lss = .lss ,
+         lscale = .lscale ,
+         lshape = .lshape ,
          zero = .zero )
-  }, list( .zero = zero, .scale.12 = scale.12, .scale.TF = scale.TF
+  }, list( .zero = zero, .scale.12 = scale.12, .scale.TF = scale.TF,
+           .lscale = lscale ,
+           .lshape = lshape ,
+           .lss = lss
          ))),
 
   initialize = eval(substitute(expression({
@@ -984,21 +1032,21 @@ if (FALSE)
 
 
     if ( .lss ) {
-      mynames1 <- paste("scale", if (ncoly > 1) 1:ncoly else "", sep = "")
-      mynames2 <- paste("shape", if (ncoly > 1) 1:ncoly else "", sep = "")
+      mynames1 <- param.names("scale", ncoly)
+      mynames2 <- param.names("shape", ncoly)
       predictors.names <-
           c(namesof(mynames1, .lscale , earg = .escale , tag = FALSE),
             namesof(mynames2, .lshape , earg = .eshape , tag = FALSE))
-            
+
     } else {
-      mynames1 <- paste("shape", if (ncoly > 1) 1:ncoly else "", sep = "")
-      mynames2 <- paste("scale", if (ncoly > 1) 1:ncoly else "", sep = "")
+      mynames1 <- param.names("shape", ncoly)
+      mynames2 <- param.names("scale", ncoly)
       predictors.names <-
           c(namesof(mynames1, .lshape , earg = .eshape , tag = FALSE),
             namesof(mynames2, .lscale , earg = .escale , tag = FALSE))
     }
     predictors.names <- predictors.names[
-          interleave.VGAM(M, M = M1)]
+          interleave.VGAM(M, M1 = M1)]
 
 
     Shape.init <- matrix(if (length( .ishape )) .ishape else 0 + NA,
@@ -1029,10 +1077,10 @@ if (FALSE)
         etastart <- if ( .lss )
           cbind(theta2eta(Scale.init, .lscale , earg = .escale ),
                 theta2eta(Shape.init, .lshape , earg = .eshape ))[,
-                interleave.VGAM(M, M = M1)] else
+                interleave.VGAM(M, M1 = M1)] else
           cbind(theta2eta(Shape.init, .lshape , earg = .eshape ),
                 theta2eta(Scale.init, .lscale , earg = .escale ))[,
-                interleave.VGAM(M, M = M1)]
+                interleave.VGAM(M, M1 = M1)]
       }
     }
   }), list( .lscale = lscale, .lshape = lshape,
@@ -1069,12 +1117,12 @@ if (FALSE)
 
 
     M1 <- extra$M1
-    avector <- if ( .lss ) c(rep( .lscale , length = ncoly),
-                             rep( .lshape , length = ncoly)) else
-                           c(rep( .lshape , length = ncoly),
-                             rep( .lscale , length = ncoly))
-    misc$link <- avector[interleave.VGAM(M, M = M1)]
-    temp.names <- c(mynames1, mynames2)[interleave.VGAM(M, M = M1)]
+    avector <- if ( .lss ) c(rep_len( .lscale , ncoly),
+                             rep_len( .lshape , ncoly)) else
+                           c(rep_len( .lshape , ncoly),
+                             rep_len( .lscale , ncoly))
+    misc$link <- avector[interleave.VGAM(M, M1 = M1)]
+    temp.names <- c(mynames1, mynames2)[interleave.VGAM(M, M1 = M1)]
     names(misc$link) <- temp.names
 
     misc$earg <- vector("list", M)
@@ -1110,6 +1158,15 @@ if (FALSE)
            .escale = escale, .eshape = eshape,
            .scale.12 = scale.12, .scale.TF = scale.TF, .lss = lss ) )),
   vfamily = c("weibullR"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    Scale <- eta2theta(eta[,    .scale.TF  ], .lscale , earg = .escale )
+    Shape <- eta2theta(eta[, !( .scale.TF )], .lshape , earg = .eshape )
+    okay1 <- all(is.finite(Scale)) && all(0 < Scale) &&
+             all(is.finite(Shape)) && all(0 < Shape)
+    okay1
+  }, list( .lscale = lscale, .lshape = lshape,
+           .escale = escale, .eshape = eshape,
+           .scale.12 = scale.12, .scale.TF = scale.TF, .lss = lss ) )),
   deriv = eval(substitute(expression({
     M1 <- 2
     Scale <- eta2theta(eta[,    .scale.TF  ], .lscale , earg = .escale )
@@ -1127,7 +1184,7 @@ if (FALSE)
                               dl.dshape * dshape.deta) else
                  c(w) * cbind(dl.dshape * dshape.deta,
                               dl.dscale * dscale.deta)
-    myderiv[, interleave.VGAM(M, M = M1)]
+    myderiv[, interleave.VGAM(M, M1 = M1)]
   }), list( .lscale = lscale, .lshape = lshape,
             .escale = escale, .eshape = eshape,
             .scale.12 = scale.12, .scale.TF = scale.TF, .lss = lss ) )),
@@ -1353,7 +1410,7 @@ pgamma.deriv.unscaled <- function(q, shape) {
 
   alld <- pgamma.deriv(q = q, shape = shape)  # 6-coln matrix
   tmp3 <- alld[, 3] / I.sq  # RHS of eqn (4.5) of \cite{wing:1989}
-    
+
   G1s <- digamma(shape) + tmp3  # eqn (4.9)
   gam1 <- gam0 * G1s
 
@@ -1379,7 +1436,8 @@ pgamma.deriv.unscaled <- function(q, shape) {
            iAlpha = NULL,   iBetaa = NULL,
            nrfs = 1,
            probs.y = c(0.2, 0.5, 0.8),
-           imethod = 1, zero = -2) {
+           imethod = 1,
+           zero = "Betaa") {
 
 
 
@@ -1398,9 +1456,6 @@ pgamma.deriv.unscaled <- function(q, shape) {
   lBetaa <- attr(eBetaa, "function.name")
 
 
-  if (length(zero) &&
-      !is.Numeric(zero, integer.valued = TRUE))
-    stop("bad input for argument 'zero'")
 
   if (!is.Numeric(imethod, length.arg = 1,
                   integer.valued = TRUE, positive = TRUE) ||
@@ -1431,24 +1486,31 @@ pgamma.deriv.unscaled <- function(q, shape) {
   new("vglmff",
   blurb = c("Truncated weibull distribution\n\n",
             "Links:    ",
-            namesof("Alpha", lAlpha, earg = eAlpha), ", ", 
-            namesof("Betaa", lBetaa, earg = eBetaa), "\n", 
+            namesof("Alpha", lAlpha, earg = eAlpha), ", ",
+            namesof("Betaa", lBetaa, earg = eBetaa), "\n",
             if (length( lower.limit ) < 5)
               paste("Truncation point(s):     ",
                     lower.limit, sep = ", ") else
               ""),
  constraints = eval(substitute(expression({
-    dotzero <- .zero
-    M1 <- 2
-    eval(negzero.expression.VGAM)
+    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M,
+                                predictors.names = predictors.names,
+                                M1 = 2)
   }), list( .zero = zero ))),
 
   infos = eval(substitute(function(...) {
     list(M1 = 2,
          Q1 = 1,
+         expected = TRUE,
+         multipleResponses = TRUE,
+         parameters.names = c("Alpha", "Betaa"),
          lower.limit = .lower.limit ,
+         lAlpha = .lAlpha ,
+         lBetaa = .lBetaa ,
          zero = .zero )
   }, list( .zero = zero,
+           .lAlpha = lAlpha ,
+           .lBetaa = lBetaa ,
            .lower.limit = lower.limit
          ))),
 
@@ -1484,12 +1546,12 @@ pgamma.deriv.unscaled <- function(q, shape) {
            "don't use SurvS4()")
 
 
-    mynames1 <- paste("Alpha", if (ncoly > 1) 1:ncoly else "", sep = "")
-    mynames2 <- paste("Betaa", if (ncoly > 1) 1:ncoly else "", sep = "")
+    mynames1 <- param.names("Alpha", ncoly)
+    mynames2 <- param.names("Betaa", ncoly)
     predictors.names <-
         c(namesof(mynames1, .lAlpha , earg = .eAlpha , tag = FALSE),
           namesof(mynames2, .lBetaa , earg = .eBetaa , tag = FALSE))[
-          interleave.VGAM(M, M = M1)]
+          interleave.VGAM(M, M1 = M1)]
 
 
     Alpha.init <- matrix(if (length( .iAlpha )) .iAlpha else 0 + NA,
@@ -1518,14 +1580,14 @@ pgamma.deriv.unscaled <- function(q, shape) {
             Alpha.init[, ilocal] <- (1 / bbb.init)^aaa.init
         }  # ilocal
       } else {
-        Alpha.init <- rep( .iAlpha , length = n)
-        Betaa.init <- rep( .iBetaa , length = n)
+        Alpha.init <- rep_len( .iAlpha , n)
+        Betaa.init <- rep_len( .iBetaa , n)
       }
 
       etastart <-
         cbind(theta2eta(Alpha.init, .lAlpha , earg = .eAlpha ),
               theta2eta(Betaa.init, .lBetaa , earg = .eBetaa ))[,
-              interleave.VGAM(M, M = M1)]
+              interleave.VGAM(M, M1 = M1)]
     }
   }), list( .lBetaa = lBetaa, .lAlpha = lAlpha,
             .eBetaa = eBetaa, .eAlpha = eAlpha,
@@ -1571,9 +1633,9 @@ pgamma.deriv.unscaled <- function(q, shape) {
 
     M1 <- extra$M1
     misc$link <-
-      c(rep( .lAlpha , length = ncoly),
-        rep( .lBetaa , length = ncoly))[interleave.VGAM(M, M = M1)]
-    temp.names <- c(mynames1, mynames2)[interleave.VGAM(M, M = M1)]
+      c(rep_len( .lAlpha , ncoly),
+        rep_len( .lBetaa , ncoly))[interleave.VGAM(M, M1 = M1)]
+    temp.names <- c(mynames1, mynames2)[interleave.VGAM(M, M1 = M1)]
     names(misc$link) <- temp.names
 
     misc$earg <- vector("list", M)
@@ -1620,6 +1682,15 @@ pgamma.deriv.unscaled <- function(q, shape) {
            .lower.limit = lower.limit ) )),
 
   vfamily = c("truncweibull"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    Alpha <- eta2theta(eta[, c(TRUE, FALSE)], .lAlpha , earg = .eAlpha )
+    Betaa <- eta2theta(eta[, c(FALSE, TRUE)], .lBetaa , earg = .eBetaa )
+    okay1 <- all(is.finite(Alpha)) && all(0 < Alpha) &&
+             all(is.finite(Betaa)) && all(0 < Betaa)
+    okay1
+  }, list( .lBetaa = lBetaa, .lAlpha = lAlpha,
+           .eBetaa = eBetaa, .eAlpha = eAlpha,
+           .lower.limit = lower.limit ) )),
 
   deriv = eval(substitute(expression({
     M1 <- 2
@@ -1639,7 +1710,7 @@ pgamma.deriv.unscaled <- function(q, shape) {
 
     myderiv <- c(w) * cbind(dl.dAlpha * dAlpha.deta,
                             dl.dBetaa * dBetaa.deta)
-    myderiv[, interleave.VGAM(M, M = M1)]
+    myderiv[, interleave.VGAM(M, M1 = M1)]
   }), list( .lBetaa = lBetaa, .lAlpha = lAlpha,
             .eBetaa = eBetaa, .eAlpha = eAlpha,
             .lower.limit = lower.limit ) )),
@@ -1652,12 +1723,12 @@ pgamma.deriv.unscaled <- function(q, shape) {
     wingo3 <- pgamma.deriv.unscaled(q = aTb,
                                     shape = 2)  # 3-cols
 
- 
+
     Eyblogy <- (exp.aTb * (digamma(2) - wingo3[, 2]) -
                (aTb + 1) * log(Alpha)) / (Alpha * Betaa)
 
 
- 
+
     Eyblog2y <- (exp.aTb * (digamma(2)^2 + trigamma(2) -
                  wingo3[, 3]) - 2 * log(Alpha) *
                 (digamma(2) - wingo3[, 2])) / (Alpha * Betaa^2) +
