@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -15,7 +15,7 @@
  SURff <-
   function(mle.normal = FALSE,
            divisor = c("n", "n-max(pj,pk)", "sqrt((n-pj)*(n-pk))"),
-           parallel = FALSE, 
+           parallel = FALSE,
            Varcov = NULL,
            matrix.arg = FALSE) {
 
@@ -59,7 +59,7 @@
   blurb = c("Seemingly unrelated regressions"),
   constraints = eval(substitute(expression({
     constraints <- cm.VGAM(matrix(1, M, 1), x = x,
-                           bool = .parallel , 
+                           bool = .parallel ,
                            constraints = constraints,
                            apply.int = .apply.parint )
   }), list( .parallel = parallel,
@@ -70,8 +70,11 @@
 
   infos = eval(substitute(function(...) {
     list(M1 = 1,  # zz???
+         Q1 = 1,
          parallel = .parallel ,
-         multipleResponses = TRUE )
+         expected = TRUE,
+         multipleResponses = TRUE,
+         parameters.names = as.character(NA))
   }, list( .parallel = parallel ))),
 
   initialize = eval(substitute(expression({
@@ -150,13 +153,13 @@
       }  # jay
     }  # !length(etastart)
   }), list(
-            .parallel = parallel 
+            .parallel = parallel
           ))),
-  linkinv = function(eta, extra = NULL) eta, 
+  linkinv = function(eta, extra = NULL) eta,
   last = eval(substitute(expression({
 
     M1 <- extra$M1
-    misc$link <- c(rep( .lmean , length = ncoly))
+    misc$link <- c(rep_len( .lmean , ncoly))
     temp.names <- predictors.names
     names(misc$link) <- temp.names
 
@@ -178,6 +181,11 @@
           ))),
 
   vfamily = "SURff",
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    mymu <- eta
+    okay1 <- all(is.finite(mymu))
+    okay1
+  }, list( .lmean = lmean, .emean = emean, .divisor = divisor ))),
 
 
   deriv = eval(substitute(expression({
@@ -197,7 +205,7 @@
         stop("argument 'divisor' unmatched"))
       Sigma.elts <- Sigma.elts * ratio.df
     } else {
-      ratio.df <- rep(1, length = M*(M+1)/2)
+      ratio.df <- rep_len(1, M*(M+1)/2)
     }
 
     Sigma.mat <- matrix(0, M, M)
@@ -217,9 +225,7 @@
     dmu.deta <- dtheta.deta(mymu,   .lmean , earg = .emean )
 
     c(w) * dl.dmu * dmu.deta
-  }), list( .lmean = lmean,
-            .emean = emean,
-            .divisor = divisor ))),
+  }), list( .lmean = lmean, .emean = emean, .divisor = divisor ))),
 
 
   weight = eval(substitute(expression({
@@ -227,7 +233,7 @@
 
     if (length( .Varcov )) {
       Sigma.mat <- if ( .matrix.arg ) .Varcov else {
-                     temp.vec <- rep( .Varcov , len = M*(M+1)/2)
+                     temp.vec <- rep_len( .Varcov , M*(M+1)/2)
                      temp.mat <- matrix(0, M, M)
                      temp.mat[cbind(iam.indices$col.index,
                                     iam.indices$row.index)] <- temp.vec
