@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -8,20 +8,25 @@
 
 
 
-vgam <- function(formula, 
-                 family, data = list(), 
-                 weights = NULL, subset = NULL, na.action = na.fail,
-                 etastart = NULL, mustart = NULL, coefstart = NULL,
-                 control = vgam.control(...),
-                 offset = NULL, 
-                 method = "vgam.fit",
-                 model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                 contrasts = NULL,
-                 constraints = NULL,
-                 extra = list(),
-                 form2 = NULL,  # Added 20130730
-                 qr.arg = FALSE, smart = TRUE, ...) {
-  dataname <- as.character(substitute(data))  # "list" if no data= 
+
+
+
+vgam <-
+  function(formula,
+           family = stop("argument 'family' needs to be assigned"),
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           etastart = NULL, mustart = NULL, coefstart = NULL,
+           control = vgam.control(...),
+           offset = NULL,
+           method = "vgam.fit",
+           model = FALSE, x.arg = TRUE, y.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = list(),
+           form2 = NULL,  # Added 20130730
+           qr.arg = FALSE, smart = TRUE, ...) {
+  dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "vgam"
 
   ocall <- match.call()
@@ -32,7 +37,10 @@ vgam <- function(formula,
   if (missing(data))
     data <- environment(formula)
 
-  mtsave <- terms(formula, "s", data = data)
+  mtsave <- terms(formula, specials = c("s", "sm.os", "sm.ps"),
+                  data = data)
+
+
 
   mf <- match.call(expand.dots = FALSE)
   m <- match(c("formula", "data", "subset", "weights", "na.action",
@@ -78,11 +86,11 @@ vgam <- function(formula,
     Xm2 <- retlist$Xm2
 
     if (length(Ym2)) {
-      if (nrow(as.matrix(Ym2)) != nrow(as.matrix(y)))
+      if (NROW(Ym2) != NROW(y))
         stop("number of rows of 'y' and 'Ym2' are unequal")
     }
     if (length(Xm2)) {
-      if (nrow(as.matrix(Xm2)) != nrow(as.matrix(x)))
+      if (NROW(Xm2) != NROW(x))
         stop("number of rows of 'x' and 'Xm2' are unequal")
     }
   } else {
@@ -104,27 +112,27 @@ vgam <- function(formula,
 
   mf2 <- mf
   if (!missing(subset)) {
-    mf2$subset <- NULL 
-    mf2 <- eval(mf2, parent.frame())  # mf2 is the full data frame. 
-    spars2 <-  lapply(mf2, attr, "spar") 
-    dfs2   <-  lapply(mf2, attr, "df") 
-    sx2 <-  lapply(mf2, attr, "s.xargument") 
-    for (ii in 1:length(mf)) {
+    mf2$subset <- NULL
+    mf2 <- eval(mf2, parent.frame())  # mf2 is the full data frame.
+    spars2 <-  lapply(mf2, attr, "spar")
+    dfs2   <-  lapply(mf2, attr, "df")
+    sx2 <-  lapply(mf2, attr, "s.xargument")
+    for (ii in seq_along(mf)) {
       if (length(sx2[[ii]])) {
         attr(mf[[ii]], "spar") <- spars2[[ii]]
         attr(mf[[ii]], "dfs2") <- dfs2[[ii]]
         attr(mf[[ii]], "s.xargument") <- sx2[[ii]]
       }
     }
-    rm(mf2) 
+    rm(mf2)
   }
 
 
 
   w <- model.weights(mf)
   if (!length(w)) {
-    w <- rep(1, nrow(mf))
-  } else if (ncol(as.matrix(w)) == 1 && any(w < 0))
+    w <- rep_len(1, nrow(mf))
+  } else if (NCOL(w) == 1 && any(w < 0))
     stop("negative weights not allowed")
 
 
@@ -142,26 +150,28 @@ vgam <- function(formula,
 
   n <- dim(x)[1]
 
-  if (FALSE && is.R()) {
-    family@linkinv <- eval(family@linkinv)
-    family@link <- eval(family@link)
-
-    for (ii in names(.min.criterion.VGAM)) 
-      if (length(family[[ii]]))
-        family[[ii]] <- eval(family[[ii]])
-  }
 
   if (length(slot(family, "first")))
     eval(slot(family, "first"))
 
-  if (method != "vgam.fit")
-    stop("method must be \"model.frame\" or \"vgam.fit\"")
-
-    # --------------------------------------------------------------
 
   aa <- attributes(mtsave)
   smoothers <- aa$specials
 
+  mgcv.sm.os <- length(smoothers$sm.os) > 0
+  mgcv.sm.ps <- length(smoothers$sm.ps) > 0
+  mgcv.sm.PS <- length(smoothers$sm.PS) > 0
+  any.sm.os.terms <- mgcv.sm.os
+  any.sm.ps.terms <- mgcv.sm.ps || mgcv.sm.PS
+  mgcv.s <- length(smoothers$s) > 0
+  if ((any.sm.os.terms || any.sm.ps.terms) && mgcv.s)
+    stop("cannot include both s() and any of sm.os() or ",
+         "sm.ps() (or sm.PS()) terms in the formula")
+  if (any.sm.os.terms && any.sm.ps.terms)
+    stop("cannot include both sm.os() and ",
+         "sm.ps() (or sm.PS()) terms in the formula")
+
+
 
 
   nonparametric <- length(smoothers$s) > 0
@@ -173,20 +183,90 @@ vgam <- function(formula,
 
     smooth.labels <- aa$term.labels[unlist(smoothers)]
   } else {
-    function.name <- "vglm"  # This is effectively so 
+    function.name <- "vglm"  # This is effectively so
   }
 
 
 
+
+
+
+
+  are.sm.os.terms <-  length(smoothers$sm.os) > 0
+  are.sm.ps.terms <- (length(smoothers$sm.ps) +
+                      length(smoothers$sm.PS)) > 0
+  if (are.sm.os.terms || are.sm.ps.terms) {
+    control$criterion <- "coefficients"  # Overwrite if necessary
+
+
+    if (length(smoothers$sm.os) > 0) {
+      ff.sm.os <- apply(aa$factors[smoothers[["sm.os"]],,drop = FALSE],
+                        2, any)
+      smoothers[["sm.os"]] <-
+        if (any(ff.sm.os))
+          seq(along = ff.sm.os)[aa$order == 1 & ff.sm.os] else NULL
+      smooth.labels <- aa$term.labels[unlist(smoothers)]
+    }
+
+    if (length(smoothers$sm.ps) > 0) {
+      ff.sm.ps <- apply(aa$factors[smoothers[["sm.ps"]],,drop = FALSE],
+                        2, any)
+      smoothers[["sm.ps"]] <-
+        if (any(ff.sm.ps))
+          seq(along = ff.sm.ps)[aa$order == 1 & ff.sm.ps] else NULL
+      smooth.labels <- aa$term.labels[unlist(smoothers)]
+    }
+
+
+
+
+
+
+    assignx <- attr(x, "assign")
+    which.X.sm.osps <- assignx[smooth.labels]
+    Data <- mf[, names(which.X.sm.osps), drop = FALSE]
+    attr(Data, "class") <- NULL
+    S.arg     <- lapply(Data, attr, "S.arg")
+    sparlist  <- lapply(Data, attr, "spar")
+    ridge.adj <- lapply(Data, attr, "ridge.adj")
+    fixspar   <- lapply(Data, attr, "fixspar")
+    ps.int    <- lapply(Data, attr, "ps.int")  # FYI only; for sm.ps()
+    knots     <- lapply(Data, attr, "knots")   # FYI only; for sm.os()
+    term.labels <- aa$term.labels
+
+
+  }
+
+
+
+  sm.osps.list <-
+    if (any.sm.os.terms || any.sm.ps.terms)
+      list(indexterms = if (any.sm.os.terms)
+                        ff.sm.os else ff.sm.ps,
+           intercept = aa$intercept,
+           which.X.sm.osps = which.X.sm.osps,
+           S.arg = S.arg,
+           sparlist = sparlist,
+           ridge.adj = ridge.adj,
+           term.labels = term.labels,
+           fixspar = fixspar,
+           orig.fixspar = fixspar,  # For posterity
+           ps.int = ps.int,  # FYI only
+           knots  = knots,   # FYI only
+           assignx = assignx) else
+      NULL
+
+
   fit <- vgam.fit(x = x, y = y, w = w, mf = mf,
                   Xm2 = Xm2, Ym2 = Ym2,  # Added 20130730
       etastart = etastart, mustart = mustart, coefstart = coefstart,
       offset = offset, family = family, control = control,
-      criterion = control$criterion,
       constraints = constraints, extra = extra, qr.arg = qr.arg,
       Terms = mtsave,
       nonparametric = nonparametric, smooth.labels = smooth.labels,
-      function.name = function.name, ...)
+      function.name = function.name,
+      sm.osps.list = sm.osps.list,
+      ...)
 
 
   if (is.Numeric(fit$nl.df) && any(fit$nl.df < 0)) {
@@ -194,16 +274,18 @@ vgam <- function(formula,
   }
 
 
+
+
   if (!is.null(fit[["smooth.frame"]])) {
-    fit <- fit[-1]       # Strip off smooth.frame
+    fit <- fit[-1]  # Strip off smooth.frame
   } else {
   }
 
-  fit$smomat <- NULL          # Not needed
+  fit$smomat <- NULL  # Not needed
 
-  fit$call <- ocall 
+  fit$call <- ocall
   if (model)
-    fit$model <- mf 
+    fit$model <- mf
   if (!x.arg)
     fit$x <- NULL
   if (!y.arg)
@@ -220,8 +302,14 @@ vgam <- function(formula,
     fit$smart.prediction <- get.smart.prediction()
 
 
+
+
+
+
   answer <-
-  new("vgam",
+  new(
+    if (any.sm.os.terms || any.sm.ps.terms) "pvgam" else "vgam",
+
     "assign"       = attr(x, "assign"),
     "call"         = fit$call,
     "coefficients" = fit$coefficients,
@@ -263,6 +351,20 @@ vgam <- function(formula,
 
 
 
+  if (length(fit$misc$Xvlm.aug)) {
+    slot(answer, "ospsslot") <-
+      list(Xvlm.aug     = fit$misc$Xvlm.aug,
+           sm.osps.list = fit$misc$sm.osps.list,
+           magicfit     = fit$misc$magicfit,
+           iter.outer   = fit$misc$iter.outer)
+    fit$misc$Xvlm.aug     <- NULL
+    fit$misc$sm.osps.list <- NULL
+    fit$misc$magicfit     <- NULL
+    fit$misc$iter.outer   <- NULL
+  }
+
+
+
   if (x.arg && length(Xm2))
     slot(answer, "Xm2") <- Xm2  # The second (lm) design matrix
   if (y.arg && length(Ym2))
@@ -319,13 +421,22 @@ vgam <- function(formula,
     slot(answer, "effects") <- fit$effects
 
 
+
+
+
+  if (nonparametric && is.buggy.vlm(answer)) {
+    warning("some s() terms have constraint matrices that have columns",
+            " which are not orthogonal;",
+            " try using sm.os() or sm.ps() instead of s().")
+  } else {
+  }
+
+
+
+
   answer
 }
-attr(vgam, "smart") <- TRUE 
-
-
-
-
+attr(vgam, "smart") <- TRUE
 
 
 
@@ -333,24 +444,24 @@ attr(vgam, "smart") <- TRUE
 
 shadowvgam <-
         function(formula,
-                 family, data = list(), 
+                 family, data = list(),
                  weights = NULL, subset = NULL, na.action = na.fail,
                  etastart = NULL, mustart = NULL, coefstart = NULL,
-                 control = vgam.control(...), 
-                 offset = NULL, 
+                 control = vgam.control(...),
+                 offset = NULL,
                  method = "vgam.fit",
                  model = FALSE, x.arg = TRUE, y.arg = TRUE,
-                 contrasts = NULL, 
+                 contrasts = NULL,
                  constraints = NULL,
-                 extra = list(), 
+                 extra = list(),
                  qr.arg = FALSE, ...) {
     dataname <- as.character(substitute(data))  # "list" if no data=
     function.name <- "shadowvgam"
 
     ocall <- match.call()
 
-    if (missing(data)) 
-        data <- environment(formula)
+    if (missing(data))
+      data <- environment(formula)
 
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "weights", "na.action",
@@ -363,7 +474,7 @@ shadowvgam <-
            stop("invalid 'method': ", method))
     mt <- attr(mf, "terms")
 
-    x <- y <- NULL 
+    x <- y <- NULL
 
     xlev <- .getXlevels(mt, mf)
     y <- model.response(mf, "any")  # model.extract(mf, "response")
@@ -372,8 +483,7 @@ shadowvgam <-
     attr(x, "assign") <- attrassigndefault(x, mt)
 
     list(Xm2 = x, Ym2 = y, call = ocall)
-}
-
+}  # shadowvgam
 
 
 
@@ -385,10 +495,10 @@ shadowvgam <-
 is.buggy.vlm <- function(object, each.term = FALSE, ...) {
 
 
-    
+
   Hk.list <- constraints(object)
   ncl <- names(Hk.list)
-  TFvec <- rep(FALSE, length = length(ncl))
+  TFvec <- rep_len(FALSE, length(ncl))
   names(TFvec) <- ncl
 
 
@@ -400,7 +510,7 @@ is.buggy.vlm <- function(object, each.term = FALSE, ...) {
     return(if (each.term) TFvec else any(TFvec))
   }
 
-  for (kay in 1:length(ncl)) {
+  for (kay in seq_along(ncl)) {
     cmat <- Hk.list[[kay]]
     if (ncol(cmat) > 1 && substring(ncl[kay], 1, 2) == "s(") {
       CMat <- crossprod(cmat)  # t(cmat) %*% cmat
@@ -426,8 +536,3 @@ setMethod("is.buggy", signature(object = "vlm"),
 
 
 
-
-
-
-
-
