@@ -1,21 +1,22 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
 
 
-vlm <- function(formula,
-                data = list(), 
-                weights = NULL, subset = NULL, na.action = na.fail,
-                prior.weights = NULL, 
-                control = vlm.control(...), 
-                method = "qr",
-                model = FALSE, x.arg = FALSE, y.arg = TRUE, qr.arg = TRUE,
-                contrasts = NULL, 
-                constraints = NULL,
-                extra = NULL, offset = NULL,  
-                smart = TRUE, ...) {
+vlm <-
+  function(formula,
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           prior.weights = NULL,
+           control = vlm.control(...),
+           method = "qr",
+           model = FALSE, x.arg = FALSE, y.arg = TRUE, qr.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = NULL, offset = NULL,
+           smart = TRUE, ...) {
   dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "vlm"
 
@@ -64,7 +65,7 @@ vlm <- function(formula,
   wz <- model.weights(mf)
 
   y <- as.matrix(y)
-  M <- ncol(as.matrix(y))
+  M <- NCOL(y)
   n <- nrow(x)
   dy <- dimnames(y)
   dy1 <- if (length(dy[[1]])) dy[[1]] else dimnames(mf)[[1]]
@@ -73,7 +74,7 @@ vlm <- function(formula,
   predictors.names <- dy2
 
   if (!length(prior.weights)) {
-    prior.weights <- rep(1, len = n)
+    prior.weights <- rep_len(1, n)
     names(prior.weights) <- dy1
   }
   if (any(prior.weights <= 0))
@@ -83,9 +84,9 @@ vlm <- function(formula,
     identity.wts <- TRUE
   } else {
     identity.wts <- FALSE
-    temp <- ncol(as.matrix(wz))
+    temp <- NCOL(wz)
     if (temp < M || temp > M*(M+1)/2)
-      stop("input 'w' must have between ", M, " and ", M*(M+1)/2, 
+      stop("input 'w' must have between ", M, " and ", M*(M+1)/2,
            " columns")
     wz <- prior.weights * wz
   }
@@ -129,25 +130,25 @@ vlm <- function(formula,
         p = ncol(x),
         ncol.X.vlm = ncol.X.vlm,
         ynames = dimnames(y)[[2]])
-    
+
     fit$misc <- misc
 
     fit$misc$dataname <- dataname
-    
 
-    
+
+
     if (smart) {
       fit$smart.prediction <- get.smart.prediction()
       wrapup.smart()
     }
 
     answer <-
-    new("vlm", 
+    new("vlm",
       "assign"       = attr(x, "assign"),
       "call"         = ocall,
       "coefficients" = fit$coefficients,
       "constraints"  = fit$constraints,
-      "control"      = control, 
+      "control"      = control,
       "criterion"    = list(deviance = fit$ResSS),
       "dispersion"   = 1,
       "df.residual"  = fit$df.residual,
@@ -190,7 +191,7 @@ vlm <- function(formula,
 
   answer
 }
-attr(vlm, "smart") <- TRUE    
+attr(vlm, "smart") <- TRUE
 
 
 
